# frozen_string_literal: true

# Copyright 2019 Richard Davis
#
# This file is part of glsnip.
#
# glsnip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glsnip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glsnip.  If not, see <http://www.gnu.org/licenses/>.

##
# = /lib/glsnip.rb
# Author::    Richard Davis
# Copyright:: Copyright 2019 Richard Davis
# License::   GNU Public License 3
#
# Main application file that loads other files.

require 'optparse'
require 'English'
require 'json'

require 'glsnip/version'
require 'glsnip/snippets'
require 'glsnip/formatter'

trap('INT') do
  puts "\nTerminating..."
  exit
end

options = {}

optparse = OptionParser.new do |opts|
  opts.banner = 'Usage: glsnip [filename(s) | options]'

  opts.on('-a', '--all', 'Lists all of the snippets for a user') do
    gl_response = Glsnip::Snippets.list

    if gl_response.success?
      puts Glsnip::Formatter.list(JSON.parse(gl_response.body))
    else
      puts Glsnip::Formatter.error(JSON.parse(gl_response.body))
    end

    exit
  end

  opts.on('-c', '--create FILE', 'Creates a new snippet given a file') do |file_name|
    puts Glsnip::Formatter.error('File does not exist') && exit unless File.exist?(file_name)

    print 'Enter a title for the snippet -> '
    title = gets.chomp

    print 'Is this snippet private? (y/n) -> '
    visibility = gets.chomp.casecmp?('y') ? 'private' : 'public'

    content = File.read(file_name)

    gl_response = Glsnip::Snippets.create(title: title,
                                          file_name: File.split(file_name).last,
                                          content: content,
                                          visibility: visibility)

    if gl_response.success?
      puts Glsnip::Formatter.create_success(JSON.parse(gl_response.body))
    else
      puts Glsnip::Formatter.error(JSON.parse(gl_response.body))
    end

    exit
  end

  opts.on('-r', '--read ID', 'Displays the contents of a snippet given an ID') do |snippet_id|
    gl_response = Glsnip::Snippets.read_raw(snippet_id)

    if gl_response.success?
      puts gl_response.body
    else
      puts Glsnip::Formatter.error(JSON.parse(gl_response.body))
    end

    exit
  end

  opts.on('-u', '--update ID', 'Updates a snippet given an ID') do |snippet_id|
    gl_read_response = Glsnip::Snippets.read(snippet_id)
    puts Glsnip::Formatter.error(JSON.parse(gl_read_response.body)) && exit unless gl_read_response.success?

    snippet_json = JSON.parse(gl_read_response.body)

    update_params = {}

    print "Current title: \"#{snippet_json['title']}\". Change? (y/n) -> "
    if gets.chomp.casecmp?('y')
      print 'Enter a title for the snippet -> '
      update_params['title'] = gets.chomp
    end

    print "Current visibility: \"#{snippet_json['visibility']}\". Change? (y/n) -> "
    if gets.chomp.casecmp?('y')
      print 'Is this snippet private? (y/n) -> '
      update_params['visibility'] = gets.chomp.casecmp?('y') ? 'private' : 'public'
    end

    print "Current file: \"#{snippet_json['file_name']}\". Change? (y/n) -> "
    if gets.chomp.casecmp?('y')
      print 'Path to file  -> '
      file_name = gets.chomp
      puts Glsnip::Formatter.error('File does not exist') && exit unless File.exist?(file_name)
      update_params['file_name'] = File.split(file_name).last
      update_params['content'] = File.read(file_name)
    end

    gl_update_response = Glsnip::Snippets.update(snippet_id, update_params)

    if gl_update_response.success?
      puts Glsnip::Formatter.update_success(JSON.parse(gl_update_response.body))
    else
      puts Glsnip::Formatter.error(JSON.parse(gl_update_response.body))
    end

    exit
  end

  opts.on('-d', '--delete ID', 'Deletes a snippet given an ID') do |snippet_id|
    gl_response = Glsnip::Snippets.delete(snippet_id)

    if gl_response.success?
      puts Glsnip::Formatter.delete_success(snippet_id)
    else
      puts Glsnip::Formatter.error(JSON.parse(gl_response.body))
    end

    exit
  end

  opts.on('-l', '--license', 'Displays the copyright notice') do
    puts "This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
"
  end

  opts.on('-w', '--warranty', 'Displays the warranty statement') do
    puts "This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"
  end

  opts.on('-v', '--version', 'Displays the program version') do
    puts "glsnip v#{Glsnip::VERSION}"
  end

  opts.on_tail('-h', '--help', 'Displays the help screen') do
    puts opts
    exit
  end
end

begin
  optparse.parse!
rescue OptionParser::InvalidOption, OptionParser::MissingArgument
  puts $ERROR_INFO.to_s
  puts 'Use -h or --help for options.'
  exit 1
end
